@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2008-2009 Pierre-Luc Beaudoin <pierre-luc@pierlux.com>
+ * Copyright (C) 2010 Jiri Techet <techet@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -25,7 +26,6 @@
 #include "champlain-tile.h"
 
 #include "champlain-enum-types.h"
-#include "champlain-map.h"
 #include "champlain-private.h"
 
 #include <math.h>
@@ -35,7 +35,7 @@
 #include <gio/gio.h>
 #include <clutter/clutter.h>
 
-G_DEFINE_TYPE (ChamplainTile, champlain_tile, G_TYPE_OBJECT)
+G_DEFINE_TYPE (ChamplainTile, champlain_tile, CLUTTER_TYPE_GROUP)
 
 #define GET_PRIVATE(o) \
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), CHAMPLAIN_TYPE_TILE, ChamplainTilePrivate))
@@ -47,28 +47,34 @@ enum
   PROP_Y,
   PROP_ZOOM_LEVEL,
   PROP_SIZE,
-  PROP_URI,
-  PROP_FILENAME,
   PROP_STATE,
-  PROP_ACTOR,
   PROP_CONTENT,
-  PROP_ETAG
+  PROP_ETAG,
+  PROP_FADE_IN
 };
 
-struct _ChamplainTilePrivate {
+enum
+{
+  /* normal signals */
+  RENDER_COMPLETE,
+  LAST_SIGNAL
+};
+
+static guint champlain_tile_signals[LAST_SIGNAL] = { 0, };
+
+struct _ChamplainTilePrivate
+{
   gint x; /* The x position on the map (in pixels) */
   gint y; /* The y position on the map (in pixels) */
   gint size; /* The tile's width and height (only support square tiles */
   gint zoom_level; /* The tile's zoom level */
 
-  gchar *uri; /* The URI where to find the tile */
   ChamplainState state; /* The tile state: loading, validation, done */
-  gchar *filename; /* The tile's cache filename */
-  ClutterActor *actor; /* An actor grouping all content actors */
   ClutterActor *content_actor; /* The actual tile actor */
+  gboolean fade_in;
 
   GTimeVal *modified_time; /* The last modified time of the cache */
-  gchar* etag; /* The HTTP ETag sent by the server */
+  gchar *etag; /* The HTTP ETag sent by the server */
 };
 
 static void
@@ -78,43 +84,47 @@ champlain_tile_get_property (GObject *object,
     GParamSpec *pspec)
 {
   ChamplainTile *self = CHAMPLAIN_TILE (object);
+
   switch (property_id)
     {
-      case PROP_X:
-        g_value_set_int (value, champlain_tile_get_x (self));
-        break;
-      case PROP_Y:
-        g_value_set_int (value, champlain_tile_get_y (self));
-        break;
-      case PROP_ZOOM_LEVEL:
-        g_value_set_int (value, champlain_tile_get_zoom_level (self));
-        break;
-      case PROP_SIZE:
-        g_value_set_uint (value, champlain_tile_get_size (self));
-        break;
-      case PROP_STATE:
-        g_value_set_enum (value, champlain_tile_get_state (self));
-        break;
-      case PROP_URI:
-        g_value_set_string (value, champlain_tile_get_uri (self));
-        break;
-      case PROP_FILENAME:
-        g_value_set_string (value, champlain_tile_get_filename (self));
-        break;
-      case PROP_ACTOR:
-        g_value_set_object (value, champlain_tile_get_actor (self));
-        break;
-      case PROP_CONTENT:
-        g_value_set_object (value, champlain_tile_get_content (self));
-        break;
-      case PROP_ETAG:
-        g_value_set_string (value, champlain_tile_get_etag (self));
-        break;
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    case PROP_X:
+      g_value_set_int (value, champlain_tile_get_x (self));
+      break;
+
+    case PROP_Y:
+      g_value_set_int (value, champlain_tile_get_y (self));
+      break;
+
+    case PROP_ZOOM_LEVEL:
+      g_value_set_int (value, champlain_tile_get_zoom_level (self));
+      break;
+
+    case PROP_SIZE:
+      g_value_set_uint (value, champlain_tile_get_size (self));
+      break;
+
+    case PROP_STATE:
+      g_value_set_enum (value, champlain_tile_get_state (self));
+      break;
+
+    case PROP_CONTENT:
+      g_value_set_object (value, champlain_tile_get_content (self));
+      break;
+
+    case PROP_ETAG:
+      g_value_set_string (value, champlain_tile_get_etag (self));
+      break;
+
+    case PROP_FADE_IN:
+      g_value_set_boolean (value, champlain_tile_get_fade_in (self));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
 }
 
+
 static void
 champlain_tile_set_property (GObject *object,
     guint property_id,
@@ -122,73 +132,74 @@ champlain_tile_set_property (GObject *object,
     GParamSpec *pspec)
 {
   ChamplainTile *self = CHAMPLAIN_TILE (object);
+
   switch (property_id)
     {
-      case PROP_X:
-        champlain_tile_set_x (self, g_value_get_int (value));
-        break;
-      case PROP_Y:
-        champlain_tile_set_y (self, g_value_get_int (value));
-        break;
-      case PROP_ZOOM_LEVEL:
-        champlain_tile_set_zoom_level (self, g_value_get_int (value));
-        break;
-      case PROP_SIZE:
-        champlain_tile_set_size (self, g_value_get_uint (value));
-        break;
-      case PROP_STATE:
-        champlain_tile_set_state (self, g_value_get_enum (value));
-        break;
-      case PROP_URI:
-        champlain_tile_set_uri (self, g_value_get_string (value));
-        break;
-      case PROP_FILENAME:
-        champlain_tile_set_filename (self, g_value_get_string (value));
-        break;
-      case PROP_CONTENT:
-        champlain_tile_set_content (self, g_value_get_object (value), FALSE);
-        break;
-      case PROP_ETAG:
-        champlain_tile_set_etag (self, g_value_get_string (value));
-        break;
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+    case PROP_X:
+      champlain_tile_set_x (self, g_value_get_int (value));
+      break;
+
+    case PROP_Y:
+      champlain_tile_set_y (self, g_value_get_int (value));
+      break;
+
+    case PROP_ZOOM_LEVEL:
+      champlain_tile_set_zoom_level (self, g_value_get_int (value));
+      break;
+
+    case PROP_SIZE:
+      champlain_tile_set_size (self, g_value_get_uint (value));
+      break;
+
+    case PROP_STATE:
+      champlain_tile_set_state (self, g_value_get_enum (value));
+      break;
+
+    case PROP_CONTENT:
+      champlain_tile_set_content (self, g_value_get_object (value));
+      break;
+
+    case PROP_ETAG:
+      champlain_tile_set_etag (self, g_value_get_string (value));
+      break;
+
+    case PROP_FADE_IN:
+      champlain_tile_set_fade_in (self, g_value_get_boolean (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
 }
 
+
 static void
 champlain_tile_dispose (GObject *object)
 {
   ChamplainTilePrivate *priv = CHAMPLAIN_TILE (object)->priv;
 
-  if (priv->actor != NULL)
+  if (priv->content_actor)
     {
-      g_object_unref (G_OBJECT (priv->actor));
-      priv->actor = NULL;
-    }
-
-  if (priv->content_actor != NULL)
-    {
-      g_object_unref (G_OBJECT (priv->content_actor));
+      g_object_unref (priv->content_actor);
       priv->content_actor = NULL;
     }
 
   G_OBJECT_CLASS (champlain_tile_parent_class)->dispose (object);
 }
 
+
 static void
 champlain_tile_finalize (GObject *object)
 {
   ChamplainTilePrivate *priv = CHAMPLAIN_TILE (object)->priv;
 
-  g_free (priv->uri);
-  g_free (priv->filename);
   g_free (priv->modified_time);
   g_free (priv->etag);
 
   G_OBJECT_CLASS (champlain_tile_parent_class)->finalize (object);
 }
 
+
 static void
 champlain_tile_class_init (ChamplainTileClass *klass)
 {
@@ -202,12 +213,12 @@ champlain_tile_class_init (ChamplainTileClass *klass)
   object_class->finalize = champlain_tile_finalize;
 
   /**
-  * ChamplainTile:x:
-  *
-  * The x position of the tile
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:x:
+   *
+   * The x position of the tile
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_X,
       g_param_spec_int ("x",
@@ -219,12 +230,12 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           G_PARAM_READWRITE));
 
   /**
-  * ChamplainTile:y:
-  *
-  * The y position of the tile
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:y:
+   *
+   * The y position of the tile
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_Y,
       g_param_spec_int ("y",
@@ -236,12 +247,12 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           G_PARAM_READWRITE));
 
   /**
-  * ChamplainTile:zoom-level:
-  *
-  * The zoom level of the tile
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:zoom-level:
+   *
+   * The zoom level of the tile
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_ZOOM_LEVEL,
       g_param_spec_int ("zoom-level",
@@ -253,12 +264,12 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           G_PARAM_READWRITE));
 
   /**
-  * ChamplainTile:size:
-  *
-  * The size of the tile in pixels
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:size:
+   *
+   * The size of the tile in pixels
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_SIZE,
       g_param_spec_uint ("size",
@@ -270,12 +281,12 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           G_PARAM_READWRITE));
 
   /**
-  * ChamplainTile:state:
-  *
-  * The state of the tile
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:state:
+   *
+   * The state of the tile
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_STATE,
       g_param_spec_enum ("state",
@@ -286,59 +297,13 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           G_PARAM_READWRITE));
 
   /**
-  * ChamplainTile:uri:
-  *
-  * The remote uri of the tile
-  *
-  * Since: 0.4
-  */
-  g_object_class_install_property (object_class,
-      PROP_URI,
-      g_param_spec_string ("uri",
-          "URI",
-          "The URI of the tile",
-          "",
-          G_PARAM_READWRITE));
-
-  /**
-  * ChamplainTile:filename:
-  *
-  * The local path of the cached tile
-  *
-  * Since: 0.4
-  */
-  g_object_class_install_property (object_class,
-      PROP_FILENAME,
-      g_param_spec_string ("filename",
-          "Filename",
-          "The filename of the tile",
-          "",
-          G_PARAM_READWRITE));
-
-  /**
-  * ChamplainTile:actor:
-  *
-  * The #ClutterActor where the tile content is rendered.  Should never change
-  * during the tile's life.
-  *
-  * Since: 0.4
-  */
-  g_object_class_install_property (object_class,
-      PROP_ACTOR,
-      g_param_spec_object ("actor",
-          "Actor",
-          "The tile's actor",
-          CLUTTER_TYPE_ACTOR,
-          G_PARAM_READABLE));
-
-  /**
-  * ChamplainTile:content:
-  *
-  * The #ClutterActor with the specific image content.  When changing this
-  * property, the new actor will be faded in.
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:content:
+   *
+   * The #ClutterActor with the specific image content.  When changing this
+   * property, the new actor will be faded in.
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_CONTENT,
       g_param_spec_object ("content",
@@ -348,14 +313,14 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           G_PARAM_READWRITE));
 
   /**
-  * ChamplainTile:etag:
-  *
-  * The tile's ETag. This information is sent by some web servers as a mean
-  * to identify if a tile has changed.  This information is saved in the cache
-  * and sent in GET queries.
-  *
-  * Since: 0.4
-  */
+   * ChamplainTile:etag:
+   *
+   * The tile's ETag. This information is sent by some web servers as a mean
+   * to identify if a tile has changed.  This information is saved in the cache
+   * and sent in GET queries.
+   *
+   * Since: 0.4
+   */
   g_object_class_install_property (object_class,
       PROP_ETAG,
       g_param_spec_string ("etag",
@@ -364,47 +329,81 @@ champlain_tile_class_init (ChamplainTileClass *klass)
           NULL,
           G_PARAM_READWRITE));
 
+  /**
+   * ChamplainTile:fade-in:
+   *
+   * Specifies whether the tile should fade in when loading
+   *
+   * Since: 0.6
+   */
+  g_object_class_install_property (object_class,
+      PROP_FADE_IN,
+      g_param_spec_boolean ("fade-in",
+          "Fade In",
+          "Tile should fade in",
+          FALSE,
+          G_PARAM_READWRITE));
+
+  /**
+   * ChamplainTile::render-complete:
+   * @self: a #ChamplainTile
+   * @calback_data: a #ChamplainRenderCallbackData struct
+   * 
+   * The #ChamplainTile::render-complete signal is emitted when rendering of the tile is
+   * completed by the renderer.
+   *
+   * Since: 0.8
+   */
+  champlain_tile_signals[RENDER_COMPLETE] =
+    g_signal_new ("render-complete", G_OBJECT_CLASS_TYPE (object_class),
+        G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+        g_cclosure_marshal_VOID__POINTER, G_TYPE_NONE,
+        1, G_TYPE_POINTER);
 }
 
+
 static void
 champlain_tile_init (ChamplainTile *self)
 {
   ChamplainTilePrivate *priv = GET_PRIVATE (self);
+
   self->priv = priv;
 
-  priv->state = CHAMPLAIN_STATE_INIT;
+  priv->state = CHAMPLAIN_STATE_NONE;
   priv->x = 0;
   priv->y = 0;
   priv->zoom_level = 0;
   priv->size = 0;
-  priv->uri = NULL;
-  priv->filename = NULL;
   priv->modified_time = NULL;
   priv->etag = NULL;
-
-  priv->actor = g_object_ref (clutter_group_new ());
-  g_object_add_weak_pointer (G_OBJECT (priv->actor), (gpointer*)&priv->actor);
+  priv->fade_in = FALSE;
 
   priv->content_actor = NULL;
 }
 
+
 /**
  * champlain_tile_new:
  *
+ * Creates an instance of #ChamplainTile.
+ *
  * Returns: a new #ChamplainTile
  *
  * Since: 0.4
  */
-ChamplainTile*
+ChamplainTile *
 champlain_tile_new (void)
 {
   return g_object_new (CHAMPLAIN_TYPE_TILE, NULL);
 }
 
+
 /**
  * champlain_tile_get_x:
  * @self: the #ChamplainTile
  *
+ * Gets the tile's x position.
+ *
  * Returns: the tile's x position
  *
  * Since: 0.4
@@ -417,10 +416,13 @@ champlain_tile_get_x (ChamplainTile *self)
   return self->priv->x;
 }
 
+
 /**
  * champlain_tile_get_y:
  * @self: the #ChamplainTile
  *
+ * Gets the tile's y position.
+ *
  * Returns: the tile's y position
  *
  * Since: 0.4
@@ -433,10 +435,13 @@ champlain_tile_get_y (ChamplainTile *self)
   return self->priv->y;
 }
 
+
 /**
  * champlain_tile_get_zoom_level:
  * @self: the #ChamplainTile
  *
+ * Gets the tile's zoom level.
+ *
  * Returns: the tile's zoom level
  *
  * Since: 0.4
@@ -449,10 +454,13 @@ champlain_tile_get_zoom_level (ChamplainTile *self)
   return self->priv->zoom_level;
 }
 
+
 /**
  * champlain_tile_get_size:
  * @self: the #ChamplainTile
  *
+ * Gets the tile's size.
+ *
  * Returns: the tile's size in pixels
  *
  * Since: 0.4
@@ -465,10 +473,13 @@ champlain_tile_get_size (ChamplainTile *self)
   return self->priv->size;
 }
 
+
 /**
  * champlain_tile_get_state:
  * @self: the #ChamplainTile
  *
+ * Gets the current state of tile loading.
+ *
  * Returns: the tile's #ChamplainState
  *
  * Since: 0.4
@@ -481,54 +492,6 @@ champlain_tile_get_state (ChamplainTile *self)
   return self->priv->state;
 }
 
-/**
- * champlain_tile_get_uri:
- * @self: the #ChamplainTile
- *
- * Returns: the tile's remote uri
- *
- * Since: 0.4
- */
-G_CONST_RETURN gchar *
-champlain_tile_get_uri (ChamplainTile *self)
-{
-  g_return_val_if_fail (CHAMPLAIN_TILE (self), NULL);
-
-  return self->priv->uri;
-}
-
-/**
- * champlain_tile_get_filename:
- * @self: the #ChamplainTile
- *
- * Returns: the tile's local filename
- *
- * Since: 0.4
- */
-G_CONST_RETURN gchar *
-champlain_tile_get_filename (ChamplainTile *self)
-{
-  g_return_val_if_fail (CHAMPLAIN_TILE (self), NULL);
-
-  return self->priv->filename;
-}
-
-/**
- * champlain_tile_get_actor:
- * @self: the #ChamplainTile
- *
- * Returns: the tile's actor.  This actor should not change during the tile's
- * lifetime. You should not unref this actor, it is owned by the tile.
- *
- * Since: 0.4
- */
-ClutterActor *
-champlain_tile_get_actor (ChamplainTile *self)
-{
-  g_return_val_if_fail (CHAMPLAIN_TILE (self), NULL);
-
-  return self->priv->actor;
-}
 
 /**
  * champlain_tile_set_x:
@@ -550,6 +513,7 @@ champlain_tile_set_x (ChamplainTile *self,
   g_object_notify (G_OBJECT (self), "x");
 }
 
+
 /**
  * champlain_tile_set_y:
  * @self: the #ChamplainTile
@@ -570,6 +534,7 @@ champlain_tile_set_y (ChamplainTile *self,
   g_object_notify (G_OBJECT (self), "y");
 }
 
+
 /**
  * champlain_tile_set_zoom_level:
  * @self: the #ChamplainTile
@@ -590,6 +555,7 @@ champlain_tile_set_zoom_level (ChamplainTile *self,
   g_object_notify (G_OBJECT (self), "zoom-level");
 }
 
+
 /**
  * champlain_tile_set_size:
  * @self: the #ChamplainTile
@@ -610,6 +576,7 @@ champlain_tile_set_size (ChamplainTile *self,
   g_object_notify (G_OBJECT (self), "size");
 }
 
+
 /**
  * champlain_tile_set_state:
  * @self: the #ChamplainTile
@@ -625,11 +592,16 @@ champlain_tile_set_state (ChamplainTile *self,
 {
   g_return_if_fail (CHAMPLAIN_TILE (self));
 
-  self->priv->state = state;
+  ChamplainTilePrivate *priv = self->priv;
 
+  if (state == priv->state)
+    return;
+
+  priv->state = state;
   g_object_notify (G_OBJECT (self), "state");
 }
 
+
 /**
  * champlain_tile_new_full:
  * @x: the x position
@@ -637,11 +609,13 @@ champlain_tile_set_state (ChamplainTile *self,
  * @size: the size in pixels
  * @zoom_level: the zoom level
  *
+ * Creates an instance of #ChamplainTile.
+ *
  * Returns: a #ChamplainTile
  *
  * Since: 0.4
  */
-ChamplainTile*
+ChamplainTile *
 champlain_tile_new_full (gint x,
     gint y,
     guint size,
@@ -651,58 +625,13 @@ champlain_tile_new_full (gint x,
       zoom_level, "size", size, NULL);
 }
 
-/**
- * champlain_tile_set_uri:
- * @self: the #ChamplainTile
- * @uri: the uri
- *
- * Sets the tile's uri
- *
- * Since: 0.4
- */
-void
-champlain_tile_set_uri (ChamplainTile *self,
-    const gchar *uri)
-{
-  g_return_if_fail (CHAMPLAIN_TILE (self));
-  g_return_if_fail (uri != NULL);
-
-  ChamplainTilePrivate *priv = self->priv;
-
-  g_free (priv->uri);
-  priv->uri = g_strdup (uri);
-
-  g_object_notify (G_OBJECT (self), "uri");
-}
-
-/**
- * champlain_tile_set_filename:
- * @self: the #ChamplainTile
- * @filename: a local path to an image
- *
- * Sets the tile's filename
- *
- * Since: 0.4
- */
-void
-champlain_tile_set_filename (ChamplainTile *self,
-    const gchar *filename)
-{
-  g_return_if_fail (CHAMPLAIN_TILE (self));
-  g_return_if_fail (filename != NULL);
-
-  ChamplainTilePrivate *priv = self->priv;
-
-  g_free (priv->filename);
-  priv->filename = g_strdup (filename);
-
-  g_object_notify (G_OBJECT (self), "filename");
-}
 
 /**
  * champlain_tile_get_modified_time:
  * @self: the #ChamplainTile
  *
+ * Gets the tile's last modified time.
+ *
  * Returns: the tile's last modified time
  *
  * Since: 0.4
@@ -715,6 +644,7 @@ champlain_tile_get_modified_time (ChamplainTile *self)
   return self->priv->modified_time;
 }
 
+
 /**
  * champlain_tile_set_modified_time:
  * @self: the #ChamplainTile
@@ -734,39 +664,16 @@ champlain_tile_set_modified_time (ChamplainTile *self,
   ChamplainTilePrivate *priv = self->priv;
 
   g_free (priv->modified_time);
-  priv->modified_time = g_memdup(time_, sizeof (GTimeVal));
+  priv->modified_time = g_memdup (time_, sizeof (GTimeVal));
 }
 
-/**
- * champlain_tile_get_modified_time_string:
- * @self: the #ChamplainTile
- *
- * Returns: the tile's modified time as a string (formated as per RFC 1123)
- *
- * Since: 0.4
- */
-gchar *
-champlain_tile_get_modified_time_string (ChamplainTile *self)
-{
-  g_return_val_if_fail (CHAMPLAIN_TILE (self), NULL);
-
-  ChamplainTilePrivate *priv = self->priv;
-
-  if (priv->modified_time == NULL)
-    return NULL;
-
-  struct tm *other_time = gmtime (&priv->modified_time->tv_sec);
-  char value [100];
-
-  strftime (value, 100, "%a, %d %b %Y %T %Z", other_time);
-
-  return g_strdup (value);
-}
 
 /**
  * champlain_tile_get_etag:
  * @self: the #ChamplainTile
  *
+ * Gets the tile's ETag.
+ *
  * Returns: the tile's ETag
  *
  * Since: 0.4
@@ -779,6 +686,7 @@ champlain_tile_get_etag (ChamplainTile *self)
   return self->priv->etag;
 }
 
+
 /**
  * champlain_tile_set_etag:
  * @self: the #ChamplainTile
@@ -801,88 +709,94 @@ champlain_tile_set_etag (ChamplainTile *self,
   g_object_notify (G_OBJECT (self), "etag");
 }
 
-typedef struct {
-  ChamplainTile *tile;
-  ClutterActor *old_actor;
-} AnimationContext;
-
-static void
-fade_in_completed (ClutterAnimation *animation,
-    ClutterActor *old_actor)
-{
-  ClutterActor *parent;
-
-  if (old_actor == NULL)
-    return;
-
-  parent = clutter_actor_get_parent (old_actor);
-
-  if (parent != NULL)
-    clutter_container_remove (CLUTTER_CONTAINER (parent), old_actor, NULL);
-
-  g_object_unref (old_actor);
-}
 
 /**
  * champlain_tile_set_content:
  * @self: the #ChamplainTile
  * @actor: the new content
- * @fade_in: if the new content should be faded in
  *
- * Sets the tile's content
+ * Sets the tile's content. To also disppay the tile, you have to call
+ * champlain_tile_display_content() in addition.
  *
  * Since: 0.4
  */
 void
 champlain_tile_set_content (ChamplainTile *self,
-    ClutterActor *actor,
-    gboolean fade_in)
+    ClutterActor *actor)
 {
   g_return_if_fail (CHAMPLAIN_TILE (self));
-  g_return_if_fail (actor != NULL);
+  g_return_if_fail (CLUTTER_ACTOR (actor));
 
   ChamplainTilePrivate *priv = self->priv;
-  ClutterActor *old_actor = NULL;
 
-  if (priv->content_actor != NULL)
-    {
-      /* it sometimes happen that the priv->content_actor has been destroyed,
-       * this assert will help determine when with no impact on the user */
-      g_assert (CLUTTER_IS_ACTOR (priv->content_actor));
+  if (priv->content_actor)
+    clutter_actor_destroy (priv->content_actor);
 
-      if (fade_in == TRUE)
-        old_actor = g_object_ref (priv->content_actor);
-      else if (priv->actor != NULL)
-        clutter_container_remove (CLUTTER_CONTAINER (priv->actor), priv->content_actor, NULL);
-
-      g_object_unref (priv->content_actor);
-    }
-
-  if (priv->actor != NULL)
-    clutter_container_add (CLUTTER_CONTAINER (priv->actor), actor, NULL);
-
-  if (fade_in == TRUE && priv->actor != NULL)
-    {
-      ClutterAnimation *animation;
-
-      clutter_actor_set_opacity (actor, 0);
-
-      animation = clutter_actor_animate (actor,
-          CLUTTER_EASE_IN_CUBIC,
-          500,
-          "opacity", 255,
-          NULL);
-
-      g_signal_connect (animation, "completed", G_CALLBACK (fade_in_completed), old_actor);
-    }
-
-  priv->content_actor = g_object_ref (actor);
+  priv->content_actor = g_object_ref_sink (actor);
 
   g_object_notify (G_OBJECT (self), "content");
 }
 
+
+static void
+fade_in_completed (G_GNUC_UNUSED ClutterAnimation *animation, ChamplainTile *self)
+{
+  if (clutter_group_get_n_children (CLUTTER_GROUP (self)) > 1)
+    clutter_actor_destroy (clutter_group_get_nth_child (CLUTTER_GROUP (self), 0));
+}
+
+
+/**
+ * champlain_tile_display_content:
+ * @self: the #ChamplainTile
+ *
+ * Displays the tile's content.
+ *
+ * Since: 0.8
+ */
+void
+champlain_tile_display_content (ChamplainTile *self)
+{
+  g_return_if_fail (CHAMPLAIN_TILE (self));
+
+  ChamplainTilePrivate *priv = self->priv;
+  ClutterAnimation *animation;
+
+  if (!priv->content_actor)
+    return;
+
+  clutter_actor_set_opacity (priv->content_actor, 0);
+  clutter_container_add_actor (CLUTTER_CONTAINER (self), priv->content_actor);
+
+  if (priv->fade_in)
+    {
+      animation = clutter_actor_animate (priv->content_actor,
+          CLUTTER_EASE_IN_CUBIC,
+          500,
+          "opacity", 255,
+          NULL);
+    }
+  else
+    {
+      animation = clutter_actor_animate (priv->content_actor,
+          CLUTTER_LINEAR,
+          150,
+          "opacity", 255,
+          NULL);
+    }
+
+  g_signal_connect (animation, "completed", G_CALLBACK (fade_in_completed), self);
+
+  g_object_unref (priv->content_actor);
+  priv->content_actor = NULL;
+}
+
+
 /**
  * champlain_tile_get_content:
+ * @self: the #ChamplainTile
+ *
+ * Gets the tile's content actor.
  *
  * Returns: the tile's content, this actor will change each time the tile's content changes.
  * You should not unref this content, it is owned by the tile.
@@ -897,3 +811,42 @@ champlain_tile_get_content (ChamplainTile *self)
   return self->priv->content_actor;
 }
 
+
+/**
+ * champlain_tile_get_fade_in:
+ * @self: the #ChamplainTile
+ *
+ * Checks whether the tile should fade in.
+ *
+ * Returns: the return value determines whether the tile should fade in when loading.
+ *
+ * Since: 0.6
+ */
+gboolean
+champlain_tile_get_fade_in (ChamplainTile *self)
+{
+  g_return_val_if_fail (CHAMPLAIN_TILE (self), FALSE);
+
+  return self->priv->fade_in;
+}
+
+
+/**
+ * champlain_tile_set_fade_in:
+ * @self: the #ChamplainTile
+ * @fade_in: determines whether the tile should fade in when loading
+ *
+ * Sets the flag determining whether the tile should fade in when loading
+ *
+ * Since: 0.6
+ */
+void
+champlain_tile_set_fade_in (ChamplainTile *self,
+    gboolean fade_in)
+{
+  g_return_if_fail (CHAMPLAIN_TILE (self));
+
+  self->priv->fade_in = fade_in;
+
+  g_object_notify (G_OBJECT (self), "fade-in");
+}
